@@ -1,4 +1,4 @@
-/* $Id: freebob_streaming.c 241 2006-06-02 10:57:02Z pieterpalmers $ */
+/* $Id: freebob_streaming.c 446 2007-10-15 18:54:13Z pieterpalmers $ */
 
 /*
  *   FreeBoB Streaming API
@@ -78,6 +78,7 @@ static int freebob_am824_xmit(char *data,
 							  int nevents, unsigned int offset, unsigned int dbc,
 							  freebob_connection_t *connection);
 			       
+int freebob_streaming_reset_playback_streams(freebob_device_t *dev);
 
 int g_verbose=0;
 
@@ -154,13 +155,13 @@ freebob_device_t *freebob_streaming_init (freebob_device_info_t *device_info, fr
 	 * but before reading the bus description as the device capabilities can change
 	 */
 
-	if(options.node_id > -1) {
-	    if (! freebob_set_samplerate(dev->fb_handle, options.node_id, options.sample_rate)) {
-		freebob_destroy_handle(dev->fb_handle);
-		free(dev);
-		printError("Failed to set samplerate...\n");
-		return NULL;
-	    }
+    if(options.node_id > -1) {
+        if (freebob_set_samplerate(dev->fb_handle, options.node_id, options.sample_rate) != 0) {
+            freebob_destroy_handle(dev->fb_handle);
+            free(dev);
+            printError("Failed to set samplerate...\n");
+            return NULL;
+        }
 
 	} else {
 	    int devices_on_bus = freebob_get_nb_devices_on_bus(dev->fb_handle);
@@ -170,7 +171,7 @@ freebob_device_t *freebob_streaming_init (freebob_device_info_t *device_info, fr
 		int node_id=freebob_get_device_node_id(dev->fb_handle, i);
 		debugPrint(DEBUG_LEVEL_STARTUP,"set samplerate for device = %d, node = %d\n", i, node_id);
 				
-		if (! freebob_set_samplerate(dev->fb_handle, node_id, options.sample_rate)) {
+		if (freebob_set_samplerate(dev->fb_handle, node_id, options.sample_rate) != 0) {
 			freebob_destroy_handle(dev->fb_handle);
 			free(dev);
 			printError("Failed to set samplerate...\n");
@@ -224,7 +225,6 @@ freebob_device_t *freebob_streaming_init (freebob_device_info_t *device_info, fr
 	
 	dev->connections=calloc(dev->nb_connections_playback+dev->nb_connections_capture, sizeof(freebob_connection_t));
 	
-	
 	for (c=0;c<dev->nb_connections_capture;c++) {
 		memcpy(&dev->connections[c].spec, libfreebob_capture_connections->connections[c], sizeof(freebob_connection_spec_t));
 		dev->connections[c].spec.direction=FREEBOB_CAPTURE;
@@ -234,6 +234,31 @@ freebob_device_t *freebob_streaming_init (freebob_device_info_t *device_info, fr
 		dev->connections[c+dev->nb_connections_capture].spec.direction=FREEBOB_PLAYBACK;
 	}
 	
+	// In order to correct for bogus midi channel numbers
+	// we redefine the location parameter to the order the 
+	// channels appear in the specification
+	int midichannel=0;
+	for (c=0;c<dev->nb_connections_capture;c++) {
+		midichannel=0;
+		int s;
+		freebob_connection_t *connection= &(dev->connections[c]);
+		for (s=0;s<connection->spec.stream_info->nb_streams;s++) {
+			if(connection->spec.stream_info->streams[s]->format == IEC61883_STREAM_TYPE_MIDI) {
+				connection->spec.stream_info->streams[s]->location=midichannel++;
+			}
+		}
+	}
+	for (c=0;c<dev->nb_connections_playback;c++) {
+		midichannel=0;
+		int s;
+		freebob_connection_t *connection= &(dev->connections[c+dev->nb_connections_capture]);
+		for (s=0;s<connection->spec.stream_info->nb_streams;s++) {
+			if(connection->spec.stream_info->streams[s]->format == IEC61883_STREAM_TYPE_MIDI) {
+				connection->spec.stream_info->streams[s]->location=midichannel++;
+			}
+		}
+	}
+
 	/* Figure out a master connection.
 	 * Either it is given in the spec libfreebob
 	 * Or it is the first connection defined (capture connections first)
@@ -739,6 +764,9 @@ int freebob_streaming_start(freebob_device_t *dev) {
 		break;
 		}
 		
+		// reset the connection, such that the buffers are empty
+		freebob_streaming_reset_connection(dev, connection);
+
 		if(connection->iso.iso_channel<0) {
 			printError("Could not do CMP for connection %d\n",i);
 			i-=1;
@@ -825,8 +853,16 @@ int freebob_streaming_start(freebob_device_t *dev) {
 		return -1;
 	}
 	
+	// reset the playback streams
+	if((err=freebob_streaming_reset_playback_streams(dev))<0) {
+ 		// TODO: cleanup
+		printError("Could not reset playback streams.\n");
+		return err;
+	}
+
 	// put nb_periods*period_size of null frames into the playback buffers
 	if((err=freebob_streaming_prefill_playback_streams(dev))<0) {
+		// TODO: cleanup
 		printError("Could not prefill playback streams.\n");
 		return err;
 	}
@@ -1988,14 +2024,14 @@ inline int freebob_streaming_decode_midi(freebob_connection_t *connection,
 		if (stream->spec.format == IEC61883_STREAM_TYPE_MIDI) {
 			/* idea:
 			spec says: current_midi_port=(dbc+j)%8;
-			=> if we start at (dbc+stream->location-1)%8 [due to location_min=1], 
+			=> if we start at (dbc+stream->location)%8, 
 			we'll start at the right event for the midi port.
 			=> if we increment j with 8, we stay at the right event.
 			*/
 			buffer=((quadlet_t *)(stream->user_buffer));
 			written=0;
 			
- 			for(j = (dbc & 0x07)+stream->spec.location-1; j < nsamples; j += 8) {
+ 			for(j = (dbc & 0x07)+stream->spec.location; j < nsamples; j += 8) {
 				target_event=(quadlet_t *)(events + ((j * connection->spec.dimension) + stream->spec.position));
 				quadlet_t sample_int=ntohl(*target_event);
   				if(IEC61883_AM824_GET_LABEL(sample_int) != IEC61883_AM824_LABEL_MIDI_NO_DATA) {
@@ -2043,19 +2079,11 @@ inline int freebob_streaming_encode_midi(freebob_connection_t *connection,
 		assert (stream);
 		assert (stream->spec.position < connection->spec.dimension);
 		assert(stream->user_buffer);
-
+		
 		if (stream->spec.format == IEC61883_STREAM_TYPE_MIDI) {
-			// first prefill the buffer with NO_DATA's on all time muxed channels
-			for(j=0; (j < nsamples); j++) {
-				target_event=(quadlet_t *)(events + ((j * connection->spec.dimension) + stream->spec.position));
-				
-				*target_event=htonl(IEC61883_AM824_SET_LABEL(0,IEC61883_AM824_LABEL_MIDI_NO_DATA));
-
-			}
-			
 			/* idea:
 				spec says: current_midi_port=(dbc+j)%8;
-				=> if we start at (dbc+stream->location-1)%8 [due to location_min=1], 
+				=> if we start at (dbc+stream->location)%8, 
 				we'll start at the right event for the midi port.
 				=> if we increment j with 8, we stay at the right event.
 			*/
@@ -2063,8 +2091,7 @@ inline int freebob_streaming_encode_midi(freebob_connection_t *connection,
 			if(stream->midi_counter<=0) { // we can send a byte
 				read=freebob_ringbuffer_read(stream->buffer, (char *)(stream->user_buffer), 1*sizeof(quadlet_t))/sizeof(quadlet_t);
 				if(read) {
-// 					j = (dbc%8)+stream->spec.location-1; 
-					j = (dbc & 0x07)+stream->spec.location-1; 
+					j = (dbc & 0x07)+stream->spec.location; 
 					target_event=(quadlet_t *)(events + ((j * connection->spec.dimension) + stream->spec.position));
 					buffer=((quadlet_t *)(stream->user_buffer));
 				
@@ -2077,7 +2104,6 @@ inline int freebob_streaming_encode_midi(freebob_connection_t *connection,
 		}
 	}
 	return 0;
-
 }
 
 
